{
  config,
  lib,
  inputs,
  pkgs,
  ...
}:
let
  cfg = config.khscodes.provisioning;
  provisioning = {
    modules = lib.mkOption {
      type = lib.types.listOf lib.types.anything;
      description = "Modules used to bring up the needed resources";
      default = [ ];
    };
    secretsSource = lib.mkOption {
      type = lib.types.enum [
        "vault"
        "bitwarden"
      ];
      description = "Where to get the secrets for the provisioning from";
      default = "vault";
    };
    variablesNeeded = lib.mkOption {
      type = lib.types.listOf lib.types.str;
      description = "Needed environment variables for the provisioning";
      default = [ ];
    };
  };
in
{
  options.khscodes.provisioning = {
    pre = provisioning;
    post = provisioning;
    preConfig = lib.mkOption {
      type = lib.types.nullOr lib.types.path;
      description = "The generated config for the pre provisioning, if any was specified";
    };
    postConfig = lib.mkOption {
      type = lib.types.nullOr lib.types.path;
      description = "The generated config for the post provisioning, if any was specified";
    };
  };

  config = {
    khscodes.provisioning.preConfig =
      if lib.lists.length cfg.pre.modules > 0 then
        inputs.terranix.lib.terranixConfiguration {
          system = pkgs.hostPlatform.system;
          modules = cfg.pre.modules;
        }
      else
        null;
    khscodes.provisioning.postConfig =
      if lib.lists.length cfg.post.modules > 0 then
        inputs.terranix.lib.terranixConfiguration {
          system = pkgs.hostPlatform.system;
          modules = cfg.post.modules;
        }
      else
        null;
  };
}
